# 
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
# 
# The Original Code is "Java-Python Extension (JPE)".
# 
# The Initial Developer of the Original Code is Frederic Bruno Giacometti.
# Portions created by Frederic Bruno Giacometti are
# Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
# 
# Contributor(s): frederic.giacometti@arakne.com
#
# Acknowledgments:
# Particular gratitude is expressed to the following parties for their
# contributing support to the development of JPE:
#     - Arakne SARL (France) and its staff (http://www.arakne.com);
#       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
#     - The European Union through its funding of the IST Alice project
#     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
#       coordinator of the IST Alice project
#     - iPhrase Technologies Inc. (Massachusetts, USA)
#

#if __debug__:
#    import gc
#    gc.disable()

"""java module

Also see:
  - the javadoc of JPE's python module in Java.
  - the README and RELEASE_NOTE files of the distribution

Support, bug reports, patches and other info
============================================

  http://www.arakne.com/jpe
  http://sourceforge.net/projects/jpe
  mailto:jpe@arakne.com
  mailto:frederic.giacometti@arakne.com

License and rights of use
=========================

JPE is subject to the Mozilla Public License Version 1.1
(http://www.mozilla.org/MPL).

Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.

Public interface
================

boolean isInitialized() --
  Is a JavaVM present as process ressource?

None initialize( PyString libraryPath, Sequence[ PyString] options) --
  Initialize a JavaVM from the given shared library, and its options
  (see JRE documentation, or 'java -help')

  Example:
    if not java.isInitialized():
        java.initialize( os.environ[ 'JRE_LIBJVM'],
                         ('-verbose:jni,gc',
                          '-Djava.library.path=%s' % os.environ[ 'JPESRCDIR'],
                          '-Djava.class.path=' + os.environ[ 'CLASSPATH'],
                          ))

java.jobject.Jclass getJclass( PyString javaClassName) --
  Java class factory
  
  Example:
     System = getJclass( 'java.lang.System')

Jstring Jstring( PyString) --
  Java string factory: creates a Java string from a Python string

  Example:
    jstr = Jstring( 'I will go to Java')

module throwable --
   Java-specific exception classes
  

Programming paradigm
====================

java.jobject.Jclass instances are java object factories;
their calling conventions exactly map those of the Java constructors
of the Java classes they emulate.

All Java static features (attributes and methods) are accessed as
the regular features of the corresponding Jclass instance.

Java objects are all java.jobject.Jobject instances; their non-static
features are accessed as the regular features of the Jobject instance.

Example:
  getJclass( 'java.lang.System').out.println( Jstring( 'Hello Java world'))


The Jobject hierarchy
=====================

In addition, Jobject has five subclasses for implementing particular protocols:
   java.jobject.{ Jstring, Jarray, Jlist, Jmap, Jstream}

   Jstring: wrapper for Java string objects
       - provides interfaces with Python string objects
   
   Jarray: wrapper for Java array objects
       - implementents the fixed-length Sequence protocol
       
   Jlist: wrapper for Java objects implementing the java.util.List interface
       - implements the Python List protocol
       
   Jmap: wrapper for Java objects implementing the java.util.Map interface
       - implements the Python dictionary protocol

   Jstream: wrapper for java.io.InputStream and java.io.OutputStream objects
       - implements the Python stream protocol (write, read, flush, close...)

Exceptions
==========
  See java.throwable.__doc__
"""

__all__ = ('getJclass', 'Jstring', 'isInitialized', 'initialize', 'throwable')

from jobject import Jobject, MakeJobject, getJclass, Jstring
from jvm import isInitialized, initialize

from javalib import JobjectType
try:
    from javalib import VoidJType, BooleanJType, ByteJType,\
         CharacterJType, ShortJType, IntegerJType, LongJType, FloatJType,\
         DoubleJType, ObjectJType
except ImportError:
    pass # will be imported in JavaVM.__init__()

