/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is "Java-Python Extension (JPE)".
 * 
 * The Initial Developer of the Original Code is Frederic Bruno Giacometti.
 * Portions created by Frederic Bruno Giacometti are
 * Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
 * 
 * Contributor(s): frederic.giacometti@arakne.com
 *
 * Acknowledgments:
 * Particular gratitude is expressed to the following parties for their
 * contributing support to the development of JPE:
 *     - Arakne SARL (France) and its staff (http://www.arakne.com);
 *       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
 *     - The European Union through its funding of the IST Alice project
 *     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
 *       coordinator of the IST Alice project
 *     - iPhrase Technologies Inc. (Massachusetts, USA)
 */

typedef PyObject* PyObject_t;
typedef char const* string_t;

#ifdef Py_DEBUG
# define JPE_DEBUG
#endif

#ifndef NOT
#define NOT !
#endif

#ifndef LogTrace
# ifdef JPE_DEBUG
#   define LogTrace\
 ((void)fprintf( stderr, "Trace@%s:%i\n", __FILE__, __LINE__))
# else
#   define LogTrace
# endif
#endif

#ifdef _WIN32
#define DLLimport __declspec( dllimport)
#else
#define DLLimport
#endif

typedef struct jarg* jarg_t;

/* javalib global functions */
extern DLLimport struct PyMethodDef jvmmodule_methods[];

int jpe_Py_Initialize( JNIEnv* env);
void jpe_initTypes( void);
int javalib_initJvm( PyObject_t module);
jobject PyJobject_AsJobject( JNIEnv* env, PyObject_t obj, jarg_t arg);
PyObject_t PyJinstance_FromJobject( JNIEnv* env, jobject jobj);
jobject jobject_fromObject( JNIEnv* env, PyObject_t obj);
int pyThread_attachJNIEnv( PyThreadState* pyThState, JNIEnv* env,
			   jboolean owned, int hasLock);
void jpe_javapyinit( JNIEnv* env, jobject threadGroup);

#define PyModJavaThrowable "java.throwable"

#ifndef JPE_JNI_VERSION
#define JPE_JNI_VERSION JNI_VERSION_1_2 /*overridable by compilation flag*/
#endif

/* single thread setup (troubleshooting) */
#ifdef JPE_SINGLETHREAD
# undef Py_BEGIN_ALLOW_THREADS
# define Py_BEGIN_ALLOW_THREADS
# undef Py_END_ALLOW_THREADS
# define Py_END_ALLOW_THREADS
# undef Py_UNBLOCK_THREADS
# define Py_UNBLOCK_THREADS
# define PyEval_RestoreThread( val) 1
# define PyEval_ReleaseThread( val) 1
# define PyEval_SaveThread() PyThreadState_Get()
#endif
