/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is "Java-Python Extension (JPE)".
 * 
 * The Initial Developer of the Original Code is Frederic Bruno Giacometti.
 * Portions created by Frederic Bruno Giacometti are
 * Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
 * 
 * Contributor(s): frederic.giacometti@arakne.com
 *
 * Acknowledgments:
 * Particular gratitude is expressed to the following parties for their
 * contributing support to the development of JPE:
 *     - Arakne SARL (France) and its staff (http://www.arakne.com);
 *       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
 *     - The European Union through its funding of the IST Alice project
 *     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
 *       coordinator of the IST Alice project
 *     - iPhrase Technologies Inc. (Massachusetts, USA)
 */

#ifdef _DEBUG
#define JUTIL_DEBUG
#endif

#ifndef NOT
#define NOT !
#endif


#ifndef LogTrace
# ifdef JUTIL_DEBUG
#   define LogTrace\
 ((void)fprintf( stderr, "Trace@%s:%i\n", __FILE__, __LINE__))
# else
#   define LogTrace
# endif
#endif


struct jcache
{
  int initialized;
  struct
  {
    jclass Throwable, PyThreadState, Object, Class,
      V, Z, B, C, S, I, J, F, D, L, String,
      List, Map, InputStream, OutputStream,
      ClassCastException, NoSuchMethodError,
      Field, Method, Constructor, Modifier;
  } class;
  struct
  {
    jmethodID printStackTrace;
  } Throwable;
  struct
  {
    jfieldID pyInterp;
    jmethodID attach;
  } PyThreadState;
  struct
  {
    jmethodID hashCode, toString;
  } Object;
  struct
  {
    jmethodID forName, forNameLoader, getClassLoader, getConstructors,
      getField, getMethods, getName, isArray;
  } Class;
  struct
  {
    jmethodID getDeclaringClass, getModifiers, getName, getType;
  } Field;
  struct
  {
    jmethodID getDeclaringClass, getModifiers, getName, getParameterTypes,
      getReturnType;
  } Method;
  struct
  {
    jmethodID getDeclaringClass, getModifiers, getName, getParameterTypes;
  } Constructor;
  struct
  {
    jmethodID isStatic;
  } Modifier;
};

struct jcache* jutil_cache_create( JNIEnv* env);
void jutil_cache_cleanup( struct jcache** pcache);

void jutil_rlock_take( JNIEnv* env, jobject rlock);
void jutil_rlock_release( JNIEnv* env, jobject rlock);
jclass jutil_local_class( JNIEnv* env, char const* name);
jclass jutil_global_class( JNIEnv* env, char const* name);
char const* jutil_UTF_from_jstring( JNIEnv* env, jstring jstr);
void jutil_UTF_cleanup( char const** pstr);
char const* jutil_class_name( JNIEnv* env, struct jcache* cache, jclass cls);
void jutil_make_global( JNIEnv* env, jobject* pref);
void jutil_throw( JNIEnv* env, jclass err, char const* msg);
void jutil_throw_illegal_state( JNIEnv* env, char const* msg);

enum jtype
{
  e_unknown = 0,
  e_void = 'V',
  e_jboolean = 'Z',
  e_jbyte = 'B',
  e_jchar = 'C',
  e_jshort = 'S',
  e_jint = 'I',
  e_jlong = 'J',
  e_jfloat = 'F',
  e_jdouble = 'D',
  e_jobject = 'L',
  e_jarray = '['
};

struct jarg
{
  enum jtype jtype;
  char const* sig;
  jclass cls;
};

#define jutil_sig2jtype( sig) (*(sig))
char const* jutil_name2sig( JNIEnv* env, char const* name);
void jutil_name2sig_cleanup( char const** psig);
