/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is "Java-Python Extension (JPE)".
 * 
 * The Initial Developer of the Original Code is Frederic Bruno Giacometti.
 * Portions created by Frederic Bruno Giacometti are
 * Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
 * 
 * Contributor(s): frederic.giacometti@arakne.com
 *
 * Acknowledgments:
 * Particular gratitude is expressed to the following parties for their
 * contributing support to the development of JPE:
 *     - Arakne SARL (France) and its staff (http://www.arakne.com);
 *       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
 *     - The European Union through its funding of the IST Alice project
 *     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
 *       coordinator of the IST Alice project
 *     - iPhrase Technologies Inc. (Massachusetts, USA)
 */

package python;

import java.util.Map;
import python.PyObject;

/** Root class providing a solution to define Python callbacks in the form
 * of Java interface implementation.
 * <br>example: application in form of
 * the Java DndListDriver class in the Dnd demo
 */
public class PyDriver
{
  /** turn on for troubleshooting */
  static public boolean trace = true;

  /** callback map */
  public Map trap;

  private PyDriver() { trap = null; }
  public PyDriver( Map vtrap)
  {
    trap = vtrap;
  }

  protected Object pythonCall( String name)
  {
    if (trace) System.out.println( "*** python.PyDriver.pythonCall( "
				   + name + ")");
    Object pyfun = trap.get( name);
    return null == pyfun ? null : ((PyObject)pyfun).call();
  }
  protected Object pythonCall( String name, Object arg)
  {
    if (trace) System.out.println( "*** python.PyDriver.pythonCall( "
				   + name + ", " + arg.toString() + ")");
    Object pyfun = trap.get( name);
    return null == pyfun ? null : ((PyObject)pyfun).call( arg);
  }
}
