/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is "Java-Python Extension (JPE)".
 * 
 * The Initial Developer of the Original Code is Frederic Bruno Giacometti.
 * Portions created by Frederic Bruno Giacometti are
 * Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
 * 
 * Contributor(s): frederic.giacometti@arakne.com
 *
 * Acknowledgments:
 * Particular gratitude is expressed to the following parties for their
 * contributing support to the development of JPE:
 *     - Arakne SARL (France) and its staff (http://www.arakne.com);
 *       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
 *     - The European Union through its funding of the IST Alice project
 *     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
 *       coordinator of the IST Alice project
 *     - iPhrase Technologies Inc. (Massachusetts, USA)
 */

package python;

import java.util.ListIterator;
import java.util.Collection;

/**
 * Construct required by the python.PySequence implementation
 * of the java.util.List interface.
 */

public class PyIterator extends PyObject implements ListIterator
{
  long position = 0;
  boolean bnext = true;

  public PyIterator( PySequence obj, int index)
  {
    super( obj.pself);
    position = index;
  }

  public boolean hasNext() { return position < size() - 1; }
  public Object next()
  {
    long pyobj;

    if (size() <= position)
      throw new java.util.NoSuchElementException( "!this.hasNext()");
    pyobj = Javapy.PySequence_GetItem( PyThreadState.get(),
					   pself, position);
    position++;
    bnext = true;
    return Javapy.ptr2object( pyobj);
  }
  public boolean hasPrevious() { return 0 < position; }
  public Object previous()
  {
    long pyobj;

    if (size() < 0)
      throw new java.util.NoSuchElementException( "!this.hasNext()");
    pyobj = Javapy.PySequence_GetItem( PyThreadState.get(),
					   pself, position);
    position--;
    bnext = false;
    return Javapy.ptr2object( pyobj);
  }
  public int nextIndex() { return (int)position + 1; }
  public int previousIndex() { return (int)position - 1; }
  public void remove()
  {
    Javapy.PySequence_DelItem( PyThreadState.get(), pself, position);
    if (0 < position) position--;
  }
  public int size()
  {
    long res;
    res = Javapy.PySequence_Length( PyThreadState.get(), pself);
    return res < Integer.MAX_VALUE ? (int)res : Integer.MAX_VALUE;
  }
  public boolean isEmpty() { return 0 != size(); }
  public void set( PyObject obj)
  {
    Javapy.PySequence_SetItem( PyThreadState.get(), pself,
				   bnext ? position - 1 : position + 1,
				   obj.pself);
  }
  public void set( Object obj) { set( asPython( obj)); }
  public boolean containsAll( Collection c)
  {
    throw new UnsupportedOperationException( "PyObject::containsAll( Collection");
  }
  public void add( Object o)
  {
    callMethod( "insert", new Object[] { asPython( position), o});
  }
  public boolean addAll( Collection c)
  {
    throw new UnsupportedOperationException( "PyObject::addAll( Collection");
  }
  public boolean addAll( int index, Collection c)
  {
    throw new UnsupportedOperationException( "PyObject::addAll( int, Collection");
  }
  public boolean removeAll( Collection c)
  {
    throw new UnsupportedOperationException( "PyObject::removeAll( Collection");
  }
  public boolean retainAll( Collection c)
  {
    throw new UnsupportedOperationException( "PyObject::retainAll( Collection");
  }

 }
