/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is "Java-Python Extension (JPE)".
 * 
 * The Initial Developer of the Original Code is Frederic Bruno Giacometti.
 * Portions created by Frederic Bruno Giacometti are
 * Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
 * 
 * Contributor(s): frederic.giacometti@arakne.com
 *
 * Acknowledgments:
 * Particular gratitude is expressed to the following parties for their
 * contributing support to the development of JPE:
 *     - Arakne SARL (France) and its staff (http://www.arakne.com);
 *       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
 *     - The European Union through its funding of the IST Alice project
 *     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
 *       coordinator of the IST Alice project
 *     - iPhrase Technologies Inc. (Massachusetts, USA)
 */

package python;

import java.util.Collection;
import java.util.List;
import java.util.Iterator;
import java.util.ListIterator;

public class PySequence extends PyObject implements List
{
  protected int iterator = 0;
  protected boolean bnext = true;

  /**
   * @param obj object to check
   * @return can 'obj' be wrapped as a PySequence object?
   */
  static public boolean check( PyObject obj)
  {
    return Javapy.PySequence_Check( PyThreadState.get(), obj.pself);
  }

  /** create a PySequence wrapper over a new Python List object */
  public PySequence()
  {
    super( Javapy.PyList_New( PyThreadState.get(), 0));
  }
  protected PySequence( long ptr)
  {
    super( ptr);
  }
  /** create a new PySequence wrapper over a Python object
   * @param: Python object on which the new wrapper is created
   * @throws java.lang.ClassCastException not a Python sequence object
   */
  public PySequence( PyObject obj)
  {
    super( obj.pself);
    if (!check( obj)) throw new ClassCastException( //should Py_DECREF?
			  "Sequence protocole not supported by "
			  + "Python object");
  }
  public PySequence( Object[] array)
  {
    super( Javapy.PyTuple_FromArray( PyThreadState.get(), array));
  }
  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public PySequence( List list)
  {
    super( 0);
    throw new UnsupportedOperationException( "PySequence::"
					     + "PySequence( List)");
  }
  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public boolean equals( Object obj)
  {
    throw new UnsupportedOperationException( "PySequence::equals( Object)");
  }
  public int size()
  {
    long res;
    res = Javapy.PySequence_Length( PyThreadState.get(), pself);
    return res < Integer.MAX_VALUE ? (int)res : Integer.MAX_VALUE;
  }
  public boolean isEmpty() { return 0 == size(); }
  public Object get( int index)
  {
    return Javapy.ptr2object( Javapy.PySequence_GetItem( PyThreadState.get(),
						  pself, index));
  }
  public Object set( int index, Object element)
  {
    Object result = get( index);
    Javapy.PySequence_SetItem( PyThreadState.get(), pself, index,
			asPython( element).pself);
    return result;
  }
  public boolean contains( PyObject o)
  {
    return Javapy.PySequence_Contains( PyThreadState.get(), pself, o.pself);
  }
  public boolean contains( Object o)
  {
    return contains( asPython( o));
  }
  public boolean append( PyObject o)
  {
    Javapy.PySequence_Append( PyThreadState.get(), pself, o.pself);
    return true;
  }
  public boolean append( Object o) { return add( asPython( o)); }

  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public boolean add( Object o)
  {
    throw new UnsupportedOperationException( "PyObject::add( Object): "
					     + "conflict between List and "
					     + "ListIterator interfaces");
  }

  public Iterator iterator() { return listIterator(); }
  public ListIterator listIterator()
  {
    return listIterator( 0);
  }
  public ListIterator listIterator( int index)
  {
    return new PyIterator( this, index);
  }

  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public Object[] toArray()
  {
    throw new UnsupportedOperationException( "PyObject::toArray()");
  }
  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public Object[] toArray( Object[] a)
  {
    throw new UnsupportedOperationException( "PyObject::toArray()");
  }

  public int indexOf( PyObject o)
  {
    return (int)Javapy.PySequence_Index( PyThreadState.get(),
					 pself, o.pself);
  }
  public int indexOf( Object o) { return indexOf( asPython( o)); }
  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public int lastIndexOf( Object o)
  {
    throw new UnsupportedOperationException( "PyObject::lastIndexOf()");
  }

  public void add( int index, Object element)
  {
    callMethod( "insert",
		new PyObject[] { asPython( index), asPython( element)});
  }
  public List subList( int fromIndex, int toIndex)
  {
    return new PySequence( Javapy.PySequence_GetSlice( PyThreadState.get(),
						       pself,
						       fromIndex, toIndex));
  }
  public Object remove( int index)
  {
    Object result = get( index);
    Javapy.PySequence_DelItem( PyThreadState.get(), pself, index);
    return result;
  }
  public boolean remove( Object obj)
  {
    int idx = indexOf( obj);
    if (0 <= idx)
      {
	remove( idx);
	return true;
      }
    else return false;
  }
  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public boolean containsAll( Collection c)
  {
    throw new UnsupportedOperationException( "PySequence::containsAll(Collection)");
  }
  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public boolean addAll( Collection c)
  {
    throw new UnsupportedOperationException( "PySequence::"
					     + "addAll(Collection)");
  }
  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public boolean addAll( int index, Collection c)
  {
    throw new UnsupportedOperationException( "PySequence::"
					     + "addAll(int,Collection)");
  }
  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public boolean removeAll( Collection c)
  {
    throw new UnsupportedOperationException( "PySequence::"
					     + "removeAll(Collection)");
  }
  /** todo
   * @throws java.lang.UnsupportedOperationException */
  public boolean retainAll( Collection c)
  {
    throw new UnsupportedOperationException( "PySequence::retainAll(Collection)");
  }
}
