# 
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
# 
# The Original Code is "Java-Python Extension (JPE)".
# 
# The Initial Developer of the Original Code is Frederic Bruno Giacometti.
# Portions created by Frederic Bruno Giacometti are
# Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
# 
# Contributor(s): frederic.giacometti@arakne.com
#
# Acknowledgments:
# Particular gratitude is expressed to the following parties for their
# contributing support to the development of JPE:
#     - Arakne SARL (France) and its staff (http://www.arakne.com);
#       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
#     - The European Union through its funding of the IST Alice project
#     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
#       coordinator of the IST Alice project
#     - iPhrase Technologies Inc. (Massachusetts, USA)
#

import os, java
from java import getJclass, Jobject, Jstring, javalib
from java.throwable import JavaIOException

if not java.isInitialized():
    jvmarg = [ '-Xcheck:jni',
               #'-verbose:jni', #,gc',
               '-Djava.library.path=' + os.environ[ 'CLASSPATH'],
               '-Djava.class.path=' + os.environ[ 'CLASSPATH'],
               ]
    print 'jvmarg =', jvmarg
    java.initialize( os.environ[ 'JRE_LIBJVM'], jvmarg)

##    r'c:\Program Files\jdk1.3\jre\bin\hotspot\jvm.dll',
##    [ r'-Djava.class.path=c:\fred\dev\jpe'])
##    [ '-Djava.class.path=' + os.environ[ 'CLASSPATH']])

System = getJclass( 'java.lang.System')
DnDConstants = getJclass( 'java.awt.dnd.DnDConstants')
DragSource = getJclass( 'java.awt.dnd.DragSource')
BoxLayout = getJclass( 'javax.swing.BoxLayout')
DataFlavor = getJclass( 'java.awt.datatransfer.DataFlavor')

if __debug__: print 'ssssss'

class DNDList( Jobject):
    def __init__( self):
        Jobject.__init__( self, getJclass( 'javax.swing.JList')()._jobject_)
        dndCallbacks = {
            'dragEnter': self.dragEnter,
            'dragExit': self.dragExit,
            'dragOver': self.dragOver,
            'drop': self.drop,
            'dragGestureRecognized': self.dragGestureRecognized,
            'dragDropEnd': self.dragDropEnd,
            'dragEnter_DragSourceDragEvent':
                self.dragEnter_DragSourceDragEvent,
            'dragExit_DragSourceEvent': self.dragExit_DragSourceEvent,
            'dragOver_DragSourceDragEvent': self.dragOver_DragSourceDragEvent,
            'dropActionChanged_DragSourceDragEvent':
                self.dropActionChanged_DragSourceDragEvent,
            }
        dndMapping = getJclass( 'python.PyMapping')( dndCallbacks)
        self.__dict__[ 'dndListDriver'] =\
                       getJclass( 'samples.DNDListDriver')( dndMapping)
        oTextField = getJclass( 'java.awt.TextField')()
        self.__dict__[ 'dropTarget'] = getJclass( 'java.awt.dnd.DropTarget')\
                                       ( oTextField, self.dndListDriver)
        self.__dict__[ 'dragSource'] = DragSource()
        self.dragSource.createDefaultDragGestureRecognizer( oTextField,
                                                DnDConstants.ACTION_MOVE,
                                                            self.dndListDriver)
    def addElement( self, s):
        self.getModel().addElement( s.toString())
    def removeElement( self):
        self.getModel().removeElement( self.getSelectedValue())
    def dragEnter( event):
        print "dragEnter"
        event.acceptDrag( DnDConstants.ACTION_MOVE)
    def dragExit( event):
        print "dragExit"
    def dragOver( event):
        print "dragOver"
    def drop( self, event):
        try:
            transferable = event.getTransferable()
            if transferable.isDataFlavorSupported( DataFlavor.stringFlavor):
                event.acceptDrop( DnDConstants.ACTION_MOVE)
                s = transferable.getTransferData( DataFlavor.stringFlavor)
                self.addElement( s)
                event.getDropTargetContext().dropComplete( 1)
            else:
                event.rejectDrop()
        except (JavaIOException, JavaUnsupportedFlavorException), excval:
            excval.printStackTrace()
            print 'Exception ' + str( excval.getMessage())
            event.rejectDrop()
    def dragGestureRecognized( self, event):
        selected = self.getSelectedValue()
        if selected:
            text = getJclass( 'java.awt.datatransfer.StringSelection')(
                                                           selected.toString())
            self.dragSource.startDrag( event, DragSource.DefaultMoveDrop,
                                       text, self.dndListDriver)
        else:
            print 'nothing was selected'
    def dragDropEnd( self, event):
        if event.getDropSuccess():
            self.removeElement()
    def dragEnter_DragSourceDragEvent( self, event):
        print 'dragEnter_DragSourceDragEvent'
    def dragExit_DragSourceEvent( self, event):
        print 'dragExit_DragSourceEvent'
    def dragOver_DragSourceDragEvent( self, event):
        print 'dragOver_DragSourceDragEvent'
    def dropActionChanged_DragSourceDragEvent( self, event):
        print 'dropActionChanged_DragSourceDragEvent'

class TestDND:
    def __init__( self):
        f = getJclass( 'javax.swing.JFrame')( Jstring( 'Drag and Drop Lists'))
        if __debug__: print 'bbb'
        sourceList = DNDList()
        sourceModel = getJclass( 'javax.swing.DefaultListModel')()
        for idx in range( 4):
            sourceModel.addElement( 'Source Item%i' % idx)
        sourcePanel = self.getListPanel( sourceList, Jstring( 'SourceList'),
                                         sourceModel)
        
        targetList = DNDList()
        targetModel = getJclass( 'javax.swing.DefaultListModel')()
        for idx in range( 4):
            targetModel.addElement( 'Target Item%i' % idx)
        targetPanel = self.getListPanel( targetList, Jstring( 'TargetList'),
                                                              targetModel)
        mainPanel = getJclass( 'javax.swing.JPanel')()
        mainPanel.setLayout( BoxLayout( mainPanel, BoxLayout.X_AXIS))
        mainPanel.add( sourcePanel)
        mainPanel.add( targetPanel)

        f.getContentPane().add( mainPanel)
        f.setSize( 300, 300)
        #eventDescr = getJclass( 'java.util.HashMap')( 1)
        #eventDescr[ 'windowClosing'] = lambda x: System.exit( 0)
        import threading
        self.__dict__[ 'exitEvent'] = threading.Event()
        def windowExit():
            if __debug__:
                print '|||| windowExit()'
            import sys
            sys.exit( 0)
        f.addWindowListener( getJclass( 'samples.DNDListDriver')\
                             ( {'windowClosing': self.windowExit}))
        f.setVisible( 1)

    def windowExit( self):
        self.exitEvent.set()

    def getListPanel( self, list, labelName, listModel):
        listPanel = getJclass( 'javax.swing.JPanel')()
        scrollPane = getJclass( 'javax.swing.JScrollPane')( list)
        list.setModel( listModel)
        nameListName = getJclass( 'javax.swing.JLabel')( labelName)
        BorderLayout = getJclass( 'java.awt.BorderLayout')
        listPanel.setLayout( BorderLayout())
        listPanel.add( nameListName, BorderLayout.NORTH)
        listPanel.add( scrollPane, BorderLayout.CENTER)
        return listPanel

print '__name__ = <%s>' % __name__

if __name__ == '__main__':
    gui = TestDND()
    if __debug__: print 'yyyy wait for exit'
    gui.exitEvent.wait()
    if __debug__: print 'zzzzz'
    import sys
    sys.exit( 0)
    
